@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 ### BEGIN LICENSE
-# Copyright (C) 2010-2012 Emilien Klein <emilien _AT_ klein _DOT_ st>
+# Copyright (C) 2010-2013 Emilien Klein <emilien _AT_ klein _DOT_ st>
 # 
 # This program is free software: you can redistribute it and/or modify it 
 # under the terms of the GNU General Public License version 3, as published 
@@ -25,6 +25,9 @@ import logging
 from nautilus_image_manipulator.helpers import get_builder
 from nautilus_image_manipulator.ImageManipulations import ImageManipulations
 from ProfileSettings import Profile, Config
+from upload.BaseUploadSite import UnknownUploadDestinationException
+from upload.BaseUploadSite import InvalidEndURLsException
+from upload.BaseUploadSite import FinalURLsNotFoundException
 
 import gettext
 from gettext import gettext as _
@@ -69,16 +72,17 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
         builder.connect_signals(self)
 
         # Populate the list of sites to upload to
-        self.upload_sites = ("1fichier.com",)
-        uploadCombo = self.o("upload_combo")
-        uploadCombo.get_model().clear()
-        for u in self.upload_sites:
-            uploadCombo.append_text(u)
+        model = self.o("upload_combo").get_model()
+        self.upload_sites = [model.get_value(model.get_iter(i), 0) for \
+                             i, k in enumerate(model)]
 
         # Load the saved configuration
         self.loadConfig()
         self.processingCanceled = False
 
+        # Give the Resize button the focus to respond to Enter
+        self.o("resize_button").grab_focus()
+
     def set_files(self, files):
         self.files = files
 
@@ -97,30 +101,6 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
         self.p = self.conf.profiles[idSelectedProfile]
         logging.info("The following profile has been selected:\n%s" % self.p)
 
-        # Check if the mandatory values are filled
-        if self.p.destination == 'folder':
-            if not self.p.foldername:
-                self.error_with_parameters(
-                    _("Please enter the name of the folder."))
-                return
-        if self.p.destination == 'append':
-            if not self.p.appendstring:
-                self.error_with_parameters(
-                    _("Please enter some text to append to the filename."))
-                return
-            elif self.p.appendstring[-1] == os.path.sep:
-                # If the appendString ends in "/", the image would be
-                # called ".EXT", which is a hidden file in it's own folder.
-                self.error_with_parameters(
-                    _("The string to append cannot end in %s") % os.path.sep)
-                return
-            # TODO: Check that the value is valid to be appended to the filename
-        if self.p.destination == 'upload':
-            if not self.p.zipname:
-                self.error_with_parameters(
-                    _("Please enter the name of the zip file."))
-                return
-
         # Disable the parameter UI elements and display the progress bar
         self.o("details_box").set_sensitive(False)
         self.o("resize_button").set_sensitive(False)
@@ -139,6 +119,36 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
         # Remember the settings for next time
         self.saveConfig()
 
+    def destination_entry_changed_cb(self, widget, data=None):
+        if widget == self.o("subfolder_entry"):
+            isError = (0 == len(widget.get_text()))
+            errorLabel = self.o("subfolder_entry_error_label")
+        elif widget == self.o("append_entry"):
+            errorLabel = None
+            if 0 == len(widget.get_text()):
+                isError = True
+                self.o("append_entry_empty_error_label").set_visible(True)
+                self.o("append_entry_invalid_error_label").set_visible(False)
+            elif os.path.sep == widget.get_text()[-1]:
+                isError = True
+                self.o("append_entry_invalid_error_label").set_text(_("The string to append cannot end in %s") % os.path.sep)
+                self.o("append_entry_empty_error_label").set_visible(False)
+                self.o("append_entry_invalid_error_label").set_visible(True)
+            else:
+                # TODO: Check that the value is valid to be appended to the filename
+                isError = False
+                self.o("append_entry_empty_error_label").set_visible(False)
+                self.o("append_entry_invalid_error_label").set_visible(False)
+        elif widget == self.o("zipname_entry"):
+            isError = (0 == len(widget.get_text()))
+            errorLabel = self.o("zipname_entry_error_label")
+
+        # Adapt the visibility of the appropriate error message
+        if errorLabel:
+            errorLabel.set_visible(isError)
+        # Don't allow resizing if text is empty
+        self.o("resize_button").set_sensitive(not isError)
+
     def on_resizing_done(self, im):
         """Triggered when all the images have been resized"""
         # Only pack and send the images if the process was not canceled and if there is at least one image to pack
@@ -163,71 +173,68 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
     def upload_file(self, im, fileToUpload):
         """Uploads a file to a website."""
         # Import the module that takes care of uploading to the selected website
-        import_string = "from upload.z%s import UploadSite" % \
-                self.p.url.replace(".", "").replace("/", "")
+        import_string = "from upload.z_%s import UploadSite" % \
+                self.p.url.replace(".", "_").replace("/", "")
         logging.debug("import_string: %s" % import_string)
         # Make sure the import does not fail
         try:
             exec import_string
         except ImportError:
-            if os.path.splitext(fileToUpload)[1] == ".zip":
-                extraInfo = _("Your images have not been sent, but have been zipped together into this file:\n%(filename)s" % {"filename": fileToUpload})
-            else:
-                extraInfo = _("Your image has not been sent, but has successfully been resized.\nYou can find it at %(filename)s" % {"filename": fileToUpload})
-            self.display_error(_("The selected upload site %(site_name)s is not valid." % {"site_name": '"%s"' % self.p.url}) + "\n\n" + extraInfo, (_("Please file a bug report on Launchpad"), "https://bugs.launchpad.net/nautilus-image-manipulator"))
+            self.error_on_uploading(_("The selected upload site %(site_name)s is not valid.") % {"site_name": '"%s"' % self.p.url} + "\n\n%(extra_info)s", fileToUpload, True)
             return
         u = None
         try:
             u = UploadSite()
         except urllib2.URLError:
             # Impossible to contact the website (no network, site down, etc.)
-            if os.path.splitext(fileToUpload)[1] == ".zip":
-                extraInfo = _("Your images have not been sent, but have been zipped together into this file:\n%(filename)s" % {"filename": fileToUpload})
-            else:
-                extraInfo = _("Your image has not been sent, but has successfully been resized.\nYou can find it at %(filename)s" % {"filename": fileToUpload})
-            self.display_error(_("The upload site %(site_name)s could not be contacted, please check your internet connection." % {"site_name": '"%s"' % self.p.url}) + "\n\n" + extraInfo)
+            self.error_on_uploading(_("The upload site %(site_name)s could not be contacted, please check your internet connection.") % {"site_name": '"%s"' % self.p.url} + "\n\n%(extra_info)s", fileToUpload, False)
+            return
+        except UnknownUploadDestinationException:
+            self.error_on_uploading(_("The upload destination for %(site_name)s could not be determined, please report a bug so that this can be fixed.") % {"site_name": '"%s"' % self.p.url} + "\n\n%(extra_info)s", fileToUpload, True)
             return
 
         self.o("progressbar").set_text("%s 0%%" % _("Uploading images..."))
         self.o("progressbar").set_fraction(0)
         self.uploadPercent = 0
-        (downloadPage, deletePage) = u.upload(fileToUpload, self.uploading_callback)
-        logging.info('downloadPage: %s' % downloadPage)
-        logging.info('deletePage: %s' % deletePage)
+        u.connect("uploading_done", self.on_uploading_done)
+        u.connect("waiting_for_validation", self.waiting_for_validation)
+        try:
+            u.upload(fileToUpload, self.uploading_callback)
+        except InvalidEndURLsException:
+            self.error_on_uploading(_("The page where your file can be downloaded from could not be determined.") + "\n" + _("Please try again, and report a bug if it happens again.") + "\n\n%(extra_info)s", fileToUpload, True)
+            return
+        except FinalURLsNotFoundException:
+            self.error_on_uploading(_("Your images were successfully uploaded to %(site_name)s, but their verification failed.") % {"site_name": '"%s"' % self.p.url} + "\n" + _("Please try again, and report a bug if it happens again.") + "\n\n%(extra_info)s", fileToUpload, True)
+            return
 
-        # Put the download url in the clipboard (both the normal "Ctrl-C" and selection clipboards)
-        # Note that the selection clipboard will be empty when the dialog gets closed.
-        # More info: http://standards.freedesktop.org/clipboards-spec/clipboards-latest.txt
-        # and http://readthedocs.org/docs/python-gtk-3-tutorial/en/latest/clipboard.html
-        Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD).set_text(downloadPage, -1)
-        Gtk.Clipboard.get(Gdk.SELECTION_PRIMARY).set_text(downloadPage, -1)
-
-        self.on_uploading_done(downloadPage, deletePage)
-
-    def display_error(self, msg, urlInfo=None):
+    def error_on_uploading(self, message, fileToUpload, reportBug):
         """Displays an error message.
         
-        Using the option ``urlInfo`` parameter, you can display a link button to open a url.
-        This parameter is a tuple of the form (message, url)"""
+        Can also display a link to report a bug on Launchpad"""
+        if os.path.splitext(fileToUpload)[1] == ".zip":
+            extra_info = _("Your images have been zipped together into this file:") + "\n" + fileToUpload
+        else:
+            extra_info = _("You can find your image at %(filename)s") % {"filename": fileToUpload}
         # Hide the unneccessary sections
         self.o("details_box").hide()
         self.o("progressbar").hide()
         self.o("url_box").hide()
         # Display the error message
-        self.o("error_message_label").set_text(msg)
+        self.o("error_message_label").set_text(message % {"extra_info": extra_info})
         self.o("error_box").show()
         # Hide the cancel and resize button, and show the close button
         self.o("cancel_button").hide()
         self.o("resize_button").hide()
         self.o("close_button").show()
-        # Eventually display an url
-        if urlInfo:
-            self.o("error_url_linkbutton").set_label(urlInfo[0])
-            self.o("error_url_linkbutton").set_uri(urlInfo[1])
+        # Display the link to report a bug
+        if reportBug:
+            self.o("error_url_linkbutton").set_label(_("Please file a bug report on Launchpad"))
+            self.o("error_url_linkbutton").set_uri("https://bugs.launchpad.net/nautilus-image-manipulator")
             self.o("error_url_hbox").show()
         else:
             self.o("error_url_hbox").hide()
-        #TODO: Make the close button the default behavior (to respond to Enter)
+        # Give the close button the focus to respond to Enter
+        self.o("close_button").grab_focus()
         self.resize(1, 1)
 
     def uploading_callback(self, param, current, total):
@@ -243,15 +250,37 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
                 Gtk.main_iteration() # Used to refresh the UI
             self.uploadPercent = percent100
 
-    def on_uploading_done(self, downloadPage, deletePage):
-        """Displays the url where the images can be downloaded from, or deleted."""
+    def pulse(self):
+        """Sets the progress bar in activity mode, and makes the block move
+        back and forth within the progress bar"""
+        self.o("progressbar").pulse()
+        return self.still_working # True = repeat, False = stop
+
+    def waiting_for_validation(self, u):
+        """Triggered when the file has been uploaded, but has not yet been verified."""
+        self.still_working = True
+        self.o("progressbar").set_text(_("Waiting for validation..."))
+        GObject.timeout_add(100, self.pulse)
+
+    def on_uploading_done(self, u):
+        """Triggered when the file has been uploaded.
+        Displays the url where the images can be downloaded from, or deleted."""
+        # Stop updating the progress bar
+        self.still_working = False
+        # Put the download url in the clipboard (both the normal "Ctrl-C" and selection clipboards)
+        # Note that the selection clipboard will be empty when the dialog gets closed.
+        # More info: http://standards.freedesktop.org/clipboards-spec/clipboards-latest.txt
+        # and http://readthedocs.org/docs/python-gtk-3-tutorial/en/latest/clipboard.html
+        Gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD).set_text(u.downloadPage, -1)
+        Gtk.Clipboard.get(Gdk.SELECTION_PRIMARY).set_text(u.downloadPage, -1)
+
         self.o("details_box").hide()
         self.o("progressbar").hide()
         # Update the link buttons with the urls
-        self.o("download_linkbutton").set_label(downloadPage)
-        self.o("download_linkbutton").set_uri(downloadPage)
-        self.o("delete_linkbutton").set_label(deletePage)
-        self.o("delete_linkbutton").set_uri(deletePage)
+        self.o("download_linkbutton").set_label(u.downloadPage)
+        self.o("download_linkbutton").set_uri(u.downloadPage)
+        self.o("delete_linkbutton").set_label(u.deletePage)
+        self.o("delete_linkbutton").set_uri(u.deletePage)
         self.o("url_box").show()
         # Hide the cancel and resize button, and show the close button
         self.o("cancel_button").hide()
@@ -325,7 +354,11 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
             self.o("subfolder_entry").set_text(p.foldername)
         elif p.destination == 'upload':
             self.o("zipname_entry").set_text(p.zipname)
-            self.o("upload_combo").set_active(self.upload_sites.index(p.url))
+            try:
+                i = self.upload_sites.index(p.url)
+            except ValueError:
+                i = 0
+            self.o("upload_combo").set_active(i)
 
     def create_new_profile_from_custom_settings(self):
         """Returns a new profile instance based on the data in the advanced
@@ -448,6 +481,8 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
         dest_model = widget.get_model()
         dest_iter = widget.get_active_iter()
         dest = dest_model.get_value(dest_iter, 1)
+        # Make sure the Resize button is clickable
+        self.o("resize_button").set_sensitive(True)
         if dest == 'folder':
             if not self.o("subfolder_entry").get_text():
                 # Default folder name
@@ -455,6 +490,7 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
             self.o("subfolder_box").show()
             self.o("append_box").hide()
             self.o("upload_box").hide()
+            self.destination_entry_changed_cb(self.o("subfolder_entry"))
         elif dest == 'append':
             if not self.o("append_entry").get_text():
                 # Default value to append to filename
@@ -462,6 +498,7 @@ class NautilusImageManipulatorDialog(Gtk.Dialog):
             self.o("subfolder_box").hide()
             self.o("append_box").show()
             self.o("upload_box").hide()
+            self.destination_entry_changed_cb(self.o("append_entry"))
         elif dest == 'upload':
             if not self.o("zipname_entry").get_text():
                 # Default zipfile name
