@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 ### BEGIN LICENSE
-# Copyright (C) 2010-2012 Emilien Klein <emilien _AT_ klein _DOT_ st>
+# Copyright (C) 2010-2013 Emilien Klein <emilien _AT_ klein _DOT_ st>
 # 
 # This program is free software: you can redistribute it and/or modify it 
 # under the terms of the GNU General Public License version 3, as published 
@@ -216,8 +216,8 @@ class Config:
         for section in sections[1:]:
             name = self.readvalue(c, section, "name")
             size = self.readvalue(c, section, "size")
-            width = self.readvalue(c, section, "width", "int")
-            height = self.readvalue(c, section, "height", "int")
+            width = self.readvalue(c, section, "width", "float")
+            height = self.readvalue(c, section, "height", "float")
             percent = self.readvalue(c, section, "percent", "float")
             quality = self.readvalue(c, section, "quality", "float", 95)
             destination = self.readvalue(c, section, "destination")
